use stm32f4xx_hal::{
    adc::{config::SampleTime, Adc},
    gpio::{Analog, Pin},
    pac::ADC1,
};

const FACTOR: f64 = 1.0 / 0.0757856;

pub struct VoltMeter {
    pin: Pin<'C', 0, Analog>,
    adc: Adc<ADC1>,
}

impl VoltMeter {
    pub fn new(pin: Pin<'C', 0, Analog>, mut adc: Adc<ADC1>) -> Self {
        adc.calibrate();
        Self { pin, adc }
    }

    pub fn voltage(&mut self) -> f64 {
        let sample = self.adc.convert(&self.pin, SampleTime::Cycles_480);
        let mv = self.adc.sample_to_millivolts(sample);

        mv as f64 * FACTOR / 1000.0
    }
}
